@@ -71,7 +71,8 @@ rasqal_iostream_write_json_boolean(raptor_iostream* iostr,
  * Return value: non-0 on failure
  **/
 static int
-rasqal_query_results_write_json1(raptor_iostream *iostr,
+rasqal_query_results_write_json1(rasqal_query_results_formatter* formatter,
+                                 raptor_iostream *iostr,
                                  rasqal_query_results* results,
                                  raptor_uri *base_uri)
 {
@@ -173,7 +174,7 @@ rasqal_query_results_write_json1(raptor_iostream *iostr,
         
         case RASQAL_LITERAL_URI:
           raptor_iostream_string_write("\"type\": \"uri\", \"value\": \"", iostr);
-          str = (const unsigned char*)raptor_uri_as_counted_string(l->value.uri, &len);
+          str = RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_counted_string(l->value.uri, &len));
           raptor_string_ntriples_write(str, len, '"', iostr);
           raptor_iostream_write_byte('"', iostr);
           break;
@@ -191,13 +192,13 @@ rasqal_query_results_write_json1(raptor_iostream *iostr,
 
           if(l->language) {
             raptor_iostream_string_write(",\n      \"xml:lang\" : \"", iostr);
-            raptor_iostream_string_write((const unsigned char*)l->language, iostr);
+            raptor_iostream_string_write(RASQAL_GOOD_CAST(const unsigned char*, l->language), iostr);
             raptor_iostream_write_byte('"', iostr);
           }
           
           if(l->datatype) {
             raptor_iostream_string_write(",\n      \"datatype\" : \"", iostr);
-            str = (const unsigned char*)raptor_uri_as_counted_string(l->datatype, &len);
+            str = RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_counted_string(l->datatype, &len));
             raptor_string_ntriples_write(str, len, '"', iostr);
             raptor_iostream_write_byte('"', iostr);
           }
@@ -213,6 +214,7 @@ rasqal_query_results_write_json1(raptor_iostream *iostr,
         case RASQAL_LITERAL_FLOAT:
         case RASQAL_LITERAL_VARIABLE:
         case RASQAL_LITERAL_DECIMAL:
+        case RASQAL_LITERAL_DATE:
         case RASQAL_LITERAL_DATETIME:
         case RASQAL_LITERAL_UDT:
         case RASQAL_LITERAL_INTEGER_SUBTYPE:
@@ -247,40 +249,51 @@ rasqal_query_results_write_json1(raptor_iostream *iostr,
 }
 
 
+static const char* const json_names[] = { "json", NULL};
+
+static const char* const json_uri_strings[] = {
+  "http://www.w3.org/ns/formats/SPARQL_Results_JSON",
+  /* W3C Working Draft */
+  "http://www.w3.org/TR/sparql11-results-json/",
+  /* W3C Working Group Note */
+  "http://www.w3.org/TR/rdf-sparql-json-res/",
+  /* Released DAWG WG results in JSON */
+  "http://www.w3.org/TR/2007/NOTE-rdf-sparql-json-res-20070618/",
+  /* URIs from 0.9.16 or earlier */
+  "http://www.w3.org/2001/sw/DataAccess/json-sparql/",
+  "http://www.mindswap.org/%7Ekendall/sparql-results-json/",
+  NULL
+};
+
+static const raptor_type_q json_types[] = {
+  { "application/sparql-results+json", 31, 10},
+  { "application/json", 16, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
+rasqal_query_results_json_register_factory(rasqal_query_results_format_factory *factory) 
+{
+  int rc = 0;
+
+  factory->desc.names = json_names;
+  factory->desc.mime_types = json_types;
+
+  factory->desc.label = "SPARQL JSON Query Results";
+  factory->desc.uri_strings = json_uri_strings;
+
+  factory->desc.flags = 0;
+  
+  factory->write         = rasqal_query_results_write_json1;
+  factory->get_rowsource = NULL;
+
+  return rc;
+}
+
+
 int
 rasqal_init_result_format_json(rasqal_world* world)
 {
-  rasqal_query_results_formatter_func writer_fn = NULL;
-  rasqal_query_results_formatter_func reader_fn = NULL;
-  rasqal_query_results_get_rowsource_func get_rowsource_fn = NULL;
-  int rc = 0;
-
-  writer_fn = &rasqal_query_results_write_json1;
-  reader_fn = NULL;
-  get_rowsource_fn = NULL;
-
-  /* Released DAWG WG results in JSON */
-  rc += rasqal_query_results_format_register_factory(world,
-                                                     "json",
-                                                     "SPARQL JSON Query Results",
-                                                     (unsigned char*)"http://www.w3.org/TR/2007/NOTE-rdf-sparql-json-res-20070618/",
-                                                     writer_fn, reader_fn, get_rowsource_fn,
-                                                     "application/json")
-                                                     != 0;
-  /* URIs from 0.9.16 or earlier */
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    NULL,
-                                                    NULL,
-                                                    (unsigned char*)"http://www.w3.org/2001/sw/DataAccess/json-sparql/",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/json")
-                                                    != 0;
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    NULL,
-                                                    NULL,
-                                                    (unsigned char*)"http://www.mindswap.org/%7Ekendall/sparql-results-json/",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/json")
-                                                    != 0;
-  return rc;
+  return !rasqal_world_register_query_results_format_factory(world,
+                                                             &rasqal_query_results_json_register_factory);
 }
