@@ -160,13 +160,8 @@ rasqal_rowsequence_rowsource_read_all_rows(rasqal_rowsource* rowsource,
   if(con->offset < 0)
     return NULL;
 
-#ifdef HAVE_RAPTOR2_API
   seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
                             (raptor_data_print_handler)rasqal_row_print);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
-                            (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
   if(seq) {
     int i;
     int size = raptor_sequence_size(con->seq);
@@ -202,7 +197,7 @@ static const rasqal_rowsource_handler rasqal_rowsequence_rowsource_handler = {
   /* .read_row = */ rasqal_rowsequence_rowsource_read_row,
   /* .read_all_rows = */ rasqal_rowsequence_rowsource_read_all_rows,
   /* .reset = */ rasqal_rowsequence_rowsource_reset,
-  /* .set_preserve = */ NULL,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ NULL,
 };
@@ -243,7 +238,7 @@ rasqal_new_rowsequence_rowsource(rasqal_world *world,
   if(!raptor_sequence_size(rows_seq) || !raptor_sequence_size(vars_seq))
     return NULL;
   
-  con = (rasqal_rowsequence_rowsource_context*)RASQAL_CALLOC(rasqal_rowsequence_rowsource_context, 1, sizeof(rasqal_rowsequence_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_rowsequence_rowsource_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
