@@ -73,9 +73,6 @@ typedef struct
   /* number of variables used in variables table  */
   int size;
   
-  /* preserve bindings when all rows are finished - for optional mostly */
-  int preserve_on_all_finished;
-
   /* GRAPH origin to use */
   rasqal_literal *origin;
 } rasqal_triples_rowsource_context;
@@ -101,13 +98,14 @@ rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
     rasqal_variable *v;
     v = rasqal_variables_table_get(rowsource->vars_table, i);
     
-    column = rasqal_query_get_bound_in_column_for_variable(query, v);
-
-    if(column >= con->start_column && column <= con->end_column) {
-      v = rasqal_new_variable_from_variable(v);
-      if(raptor_sequence_push(rowsource->variables_sequence, v))
-        return -1;
-      con->size++;
+    for(column = con->start_column; column <= con->end_column; column++) {
+      if(rasqal_query_variable_bound_in_triple(query, v, column)) {
+          v = rasqal_new_variable_from_variable(v);
+          if(raptor_sequence_push(rowsource->variables_sequence, v))
+            return -1;
+          con->size++;
+          break; /* end column search loop */
+        }
     }
   }
 
@@ -125,15 +123,15 @@ rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
     t = (rasqal_triple*)raptor_sequence_get_at(con->triples, column);
     
     if((v = rasqal_literal_as_variable(t->subject)) &&
-       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_SUBJECT)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_SUBJECT);
     
     if((v = rasqal_literal_as_variable(t->predicate)) &&
-       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_PREDICATE)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_PREDICATE);
     
     if((v = rasqal_literal_as_variable(t->object)) &&
-       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_OBJECT)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_OBJECT);
 
     RASQAL_DEBUG4("triple pattern column %d has parts %s (%d)\n", column,
@@ -221,30 +219,6 @@ rasqal_triples_rowsource_get_next_row(rasqal_rowsource* rowsource,
       RASQAL_DEBUG2("end of pattern triples match for column %d\n",
                     con->column);
 
-#if 0
-      if(con->preserve_on_all_finished && 
-         con->column == con->end_column) {
-        int is_finished = 1;
-        int i;
-          
-        RASQAL_DEBUG1("CHECKING ALL TRIPLE PATTERNS FINISHED\n");
-
-        for(i = con->start_column; i <= con->end_column; i++) {
-          rasqal_triple_meta *m2;
-          m2 = &con->triple_meta[con->column - con->start_column];
-          if(!rasqal_triples_match_is_end(m2->triples_match)) {
-            is_finished = 0;
-            break;
-          }
-        }
-        if(is_finished) {
-          RASQAL_DEBUG1("end of all pattern triples matches\n");
-          error = RASQAL_ENGINE_FINISHED;
-          break;
-        }
-      }
-#endif
-
       /* reset this column and move to next match in previous column */
       rasqal_reset_triple_meta(m);
       con->column--;
@@ -346,11 +320,8 @@ static raptor_sequence*
 rasqal_triples_rowsource_read_all_rows(rasqal_rowsource* rowsource,
                                        void *user_data)
 {
-  rasqal_triples_rowsource_context *con;
   raptor_sequence *seq = NULL;
   
-  con = (rasqal_triples_rowsource_context*)user_data;
-
   return seq;
 }
 
@@ -375,19 +346,6 @@ rasqal_triples_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_triples_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                      void *user_data, int preserve)
-{
-  rasqal_triples_rowsource_context *con;
-
-  con = (rasqal_triples_rowsource_context*)user_data;
-  con->preserve_on_all_finished = preserve;
-
-  return 0;
-}
-
-
 static int
 rasqal_triples_rowsource_set_origin(rasqal_rowsource *rowsource,
                                     void *user_data,
@@ -422,7 +380,7 @@ static const rasqal_rowsource_handler rasqal_triples_rowsource_handler = {
   /* .read_row = */ rasqal_triples_rowsource_read_row,
   /* .read_all_rows = */ rasqal_triples_rowsource_read_all_rows,
   /* .reset = */ rasqal_triples_rowsource_reset,
-  /* .set_preserve = */ rasqal_triples_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ rasqal_triples_rowsource_set_origin
 };
@@ -458,7 +416,7 @@ rasqal_new_triples_rowsource(rasqal_world *world,
   if(!triples)
     return rasqal_new_empty_rowsource(world, query);
   
-  con = (rasqal_triples_rowsource_context*)RASQAL_CALLOC(rasqal_triples_rowsource_context, 1, sizeof(rasqal_triples_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_triples_rowsource_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
@@ -470,9 +428,7 @@ rasqal_new_triples_rowsource(rasqal_world *world,
 
   con->triples_count = con->end_column - con->start_column + 1;
 
-  con->triple_meta = (rasqal_triple_meta*)RASQAL_CALLOC(rasqal_triple_meta,
-                                                        con->triples_count,
-                                                        sizeof(rasqal_triple_meta));
+  con->triple_meta = RASQAL_CALLOC(rasqal_triple_meta*, con->triples_count, sizeof(rasqal_triple_meta));
   if(!con->triple_meta) {
     rasqal_triples_rowsource_finish(NULL, con);
     return NULL;
@@ -522,8 +478,8 @@ main(int argc, char *argv[])
   unsigned char *data_string = NULL;
   unsigned char *uri_string = NULL;
   /* <http://example.org#subject> <http://example.org#predicate> "object" . */
-#define SUBJECT_URI_STRING (const unsigned char*)"http://example.org#subject"
-#define PREDICATE_URI_STRING (const unsigned char*)"http://example.org#predicate"
+#define SUBJECT_URI_STRING RASQAL_GOOD_CAST(const unsigned char*, "http://example.org#subject")
+#define PREDICATE_URI_STRING RASQAL_GOOD_CAST(const unsigned char*, "http://example.org#predicate")
 #define OBJECT_STRING "object"
   raptor_uri* s_uri = NULL;
   raptor_uri* p_uri = NULL;
@@ -548,8 +504,8 @@ main(int argc, char *argv[])
   query = rasqal_new_query(world, "sparql", NULL);
   
   data_string = raptor_uri_filename_to_uri_string(data_file);
-  query_string = (unsigned char*)RASQAL_MALLOC(cstring, strlen((const char*)data_string)+strlen(query_format)+1);
-  sprintf((char*)query_string, query_format, data_string);
+  query_string = RASQAL_MALLOC(unsigned char*, strlen(RASQAL_GOOD_CAST(const char*, data_string)) + strlen(query_format) + 1);
+  sprintf(RASQAL_GOOD_CAST(char*, query_string), query_format, data_string);
   raptor_free_memory(data_string);
   
   uri_string = raptor_uri_filename_to_uri_string("");
@@ -573,7 +529,7 @@ main(int argc, char *argv[])
     goto tidy;
   }
   
-  RASQAL_FREE(cstring, query_string);
+  RASQAL_FREE(char*, query_string);
   query_string = NULL;
 
   triples = rasqal_query_get_triple_sequence(query);
@@ -630,7 +586,7 @@ main(int argc, char *argv[])
     o = row->values[2];
     if(!o ||
        (o && o->type != RASQAL_LITERAL_STRING) ||
-       strcmp((const char*)o->string, OBJECT_STRING)) {
+       strcmp(RASQAL_GOOD_CAST(const char*, o->string), OBJECT_STRING)) {
       fprintf(stderr, "%s: 'o' is bound to %s not string '%s'\n", program,
               rasqal_literal_as_string(o), OBJECT_STRING);
       failures++;
